@@ -3,27 +3,37 @@ import os
 from dataclasses import dataclass
 from datetime import datetime
 from io import BytesIO
-from typing import Optional, Dict, List, Callable
+from typing import Callable, Dict, List, Optional
 
 import tzlocal
-from asn1crypto import ocsp, tsp, pem
-from werkzeug.wrappers import Request, Response
+from asn1crypto import ocsp, pem, tsp
 from dateutil.parser import parse as parse_dt
-from werkzeug.routing import Map, Rule, BaseConverter, Submount
-from werkzeug.exceptions import HTTPException, NotFound, InternalServerError, \
-    BadRequest
+from werkzeug.exceptions import (
+    BadRequest,
+    HTTPException,
+    InternalServerError,
+    NotFound,
+)
+from werkzeug.routing import BaseConverter, Map, Rule, Submount
+from werkzeug.wrappers import Request, Response
 
 from certomancer.config_utils import ConfigurationError
 from certomancer.crypto_utils import pyca_cryptography_present
 from certomancer.registry import (
-    PKIArchitecture, ServiceLabel, CertLabel,
-    CertomancerObjectNotFoundError, CertomancerConfig, ArchLabel, EntityLabel,
+    ArchLabel,
     AttributeCertificateSpec,
-    CertificateSpec, PluginLabel, PluginServiceRequestError
+    CertificateSpec,
+    CertLabel,
+    CertomancerConfig,
+    CertomancerObjectNotFoundError,
+    EntityLabel,
+    PKIArchitecture,
+    PluginLabel,
+    PluginServiceRequestError,
+    ServiceLabel,
 )
 from certomancer.services import CertomancerServiceError
 
-
 logger = logging.getLogger(__name__)
 
 pfx_possible = pyca_cryptography_present()
@@ -37,7 +47,6 @@ FAKE_TIME_HEADER = 'X-Certomancer-Fake-Time'
 
 
 class PemExtensionConverter(BaseConverter):
-
     def __init__(self, map, exts=('crt', 'cert', 'cer')):
         if isinstance(exts, str):
             exts = (exts,)
@@ -60,12 +69,12 @@ class AnimatorCertInfo:
 
     @staticmethod
     def gather_cert_info(pki_arch: PKIArchitecture):
-
         def _for_cert(spec: CertificateSpec):
             pfx = pfx_possible and pki_arch.is_subject_key_available(spec.label)
             return AnimatorCertInfo(
-                spec=spec, pfx_available=pfx,
-                subject_dn=pki_arch.entities[spec.subject].human_friendly
+                spec=spec,
+                pfx_available=pfx,
+                subject_dn=pki_arch.entities[spec.subject].human_friendly,
             )
 
         return {
@@ -81,11 +90,10 @@ class AnimatorAttrCertInfo:
 
     @staticmethod
     def gather_cert_info(pki_arch: PKIArchitecture):
-
         def _for_attr_cert(spec: AttributeCertificateSpec):
             return AnimatorAttrCertInfo(
                 spec=spec,
-                holder_dn=pki_arch.entities[spec.holder.name].human_friendly
+                holder_dn=pki_arch.entities[spec.holder.name].human_friendly,
             )
 
         return {
@@ -118,7 +126,7 @@ class ArchServicesDescription:
             cert_repo=services.list_cert_repos(),
             attr_cert_repo=services.list_attr_cert_repos(),
             certs_by_issuer=cert_info,
-            attr_certs_by_issuer=attr_cert_info
+            attr_certs_by_issuer=attr_cert_info,
         )
 
 
@@ -128,22 +136,40 @@ WEB_UI_URL_PREFIX = '_certomancer'
 def web_ui_rules():
     return [
         Rule('/', endpoint='index', methods=('GET',)),
-        Submount("/" + WEB_UI_URL_PREFIX, [
-            # convenience endpoint that serves certs without regard for
-            # checking whether they belong to any particular (logical)
-            # cert repo (these URLs aren't part of the "PKI API", for lack
-            # of a better term)
-            Rule('/any-cert/<arch>/<label>.<ext:use_pem>',
-                 endpoint='any-cert', methods=('GET',)),
-            Rule('/any-attr-cert/<arch>/<label>.attr.<ext:use_pem>',
-                 endpoint='any-attr-cert', methods=('GET',)),
-            Rule('/attr-certs-of/<arch>/<entity_label>-all.attr.cert.pem',
-                 endpoint='attr-certs-of', methods=('GET',)),
-            Rule('/cert-bundle/<arch>', endpoint='cert-bundle',
-                 methods=('GET',)),
-            Rule('/pfx-download/<arch>', endpoint='pfx-download',
-                 methods=('POST',)),
-        ])
+        Submount(
+            "/" + WEB_UI_URL_PREFIX,
+            [
+                # convenience endpoint that serves certs without regard for
+                # checking whether they belong to any particular (logical)
+                # cert repo (these URLs aren't part of the "PKI API", for lack
+                # of a better term)
+                Rule(
+                    '/any-cert/<arch>/<label>.<ext:use_pem>',
+                    endpoint='any-cert',
+                    methods=('GET',),
+                ),
+                Rule(
+                    '/any-attr-cert/<arch>/<label>.attr.<ext:use_pem>',
+                    endpoint='any-attr-cert',
+                    methods=('GET',),
+                ),
+                Rule(
+                    '/attr-certs-of/<arch>/<entity_label>-all.attr.cert.pem',
+                    endpoint='attr-certs-of',
+                    methods=('GET',),
+                ),
+                Rule(
+                    '/cert-bundle/<arch>',
+                    endpoint='cert-bundle',
+                    methods=('GET',),
+                ),
+                Rule(
+                    '/pfx-download/<arch>',
+                    endpoint='pfx-download',
+                    methods=('POST',),
+                ),
+            ],
+        ),
     ]
 
 
@@ -154,30 +180,56 @@ def service_rules():
         # Time stamping service pattern
         Rule('/<arch>/tsa/<label>', endpoint='tsa', methods=('POST',)),
         # Plugin endpoint pattern
-        Rule('/<arch>/plugin/<plugin_label>/<label>', endpoint='plugin',
-             methods=('POST',)),
+        Rule(
+            '/<arch>/plugin/<plugin_label>/<label>',
+            endpoint='plugin',
+            methods=('POST',),
+        ),
         # latest CRL pattern
-        Rule("/<arch>/crls/<label>/latest.<ext(exts='crl'):use_pem>",
-             endpoint='crls', methods=('GET',), defaults={'crl_no': None}),
+        Rule(
+            "/<arch>/crls/<label>/latest.<ext(exts='crl'):use_pem>",
+            endpoint='crls',
+            methods=('GET',),
+            defaults={'crl_no': None},
+        ),
         # CRL archive pattern
-        Rule("/<arch>/crls/<label>"
-             "/archive-<int:crl_no>.<ext(exts='crl'):use_pem>",
-             endpoint='crls', methods=('GET',)),
+        Rule(
+            "/<arch>/crls/<label>"
+            "/archive-<int:crl_no>.<ext(exts='crl'):use_pem>",
+            endpoint='crls',
+            methods=('GET',),
+        ),
         # Cert repo authority pattern
-        Rule('/<arch>/certs/<label>/ca.<ext:use_pem>',
-             defaults={'cert_label': None}, endpoint='certs', methods=('GET',)),
+        Rule(
+            '/<arch>/certs/<label>/ca.<ext:use_pem>',
+            defaults={'cert_label': None},
+            endpoint='certs',
+            methods=('GET',),
+        ),
         # Cert repo generic pattern
-        Rule('/<arch>/certs/<label>/issued/<cert_label>.<ext:use_pem>',
-             endpoint='certs', methods=('GET',)),
+        Rule(
+            '/<arch>/certs/<label>/issued/<cert_label>.<ext:use_pem>',
+            endpoint='certs',
+            methods=('GET',),
+        ),
         # Attr cert repo authority pattern
-        Rule('/<arch>/attr-certs/<label>/aa.<ext:use_pem>',
-             defaults={'cert_label': None},
-             endpoint='attr-certs', methods=('GET',)),
+        Rule(
+            '/<arch>/attr-certs/<label>/aa.<ext:use_pem>',
+            defaults={'cert_label': None},
+            endpoint='attr-certs',
+            methods=('GET',),
+        ),
         # Attr cert repo generic pattern
-        Rule("/<arch>/attr-certs/<label>/issued/<cert_label>.attr.<ext:use_pem>",
-             endpoint='attr-certs', methods=('GET',)),
-        Rule("/<arch>/attr-certs/<label>/by-holder/<entity_label>-all.attr.cert.pem",
-             endpoint='attr-certs-by-holder', methods=('GET',)),
+        Rule(
+            "/<arch>/attr-certs/<label>/issued/<cert_label>.attr.<ext:use_pem>",
+            endpoint='attr-certs',
+            methods=('GET',),
+        ),
+        Rule(
+            "/<arch>/attr-certs/<label>/by-holder/<entity_label>-all.attr.cert.pem",
+            endpoint='attr-certs-by-holder',
+            methods=('GET',),
+        ),
     ]
 
 
@@ -193,19 +245,19 @@ def gen_index(architectures):
     #  just go ahead and render it
     jinja_env = Environment(
         loader=PackageLoader('certomancer.integrations', 'animator_templates'),
-        autoescape=True
+        autoescape=True,
     )
     template = jinja_env.get_template('index.html')
     return template.render(
         pki_archs=[
             ArchServicesDescription.compile(arch) for arch in architectures
         ],
-        pfx_possible=pfx_possible, web_ui_prefix=WEB_UI_URL_PREFIX
+        pfx_possible=pfx_possible,
+        web_ui_prefix=WEB_UI_URL_PREFIX,
     )
 
 
 class AnimatorArchStore:
-
     def __init__(self, architectures: Dict[ArchLabel, PKIArchitecture]):
         self.architectures = architectures
 
@@ -220,19 +272,21 @@ class AnimatorArchStore:
 
 
 class Animator:
-
-    def __init__(self, architectures: AnimatorArchStore,
-                 at_time: Optional[datetime] = None, with_web_ui=True,
-                 allow_time_override=True):
+    def __init__(
+        self,
+        architectures: AnimatorArchStore,
+        at_time: Optional[datetime] = None,
+        with_web_ui=True,
+        allow_time_override=True,
+    ):
         self.fixed_time = at_time
         self.architectures = architectures
         self.with_web_ui = with_web_ui
-        self.url_map = None
         self.allow_time_override = allow_time_override
 
         self.url_map = Map(
             service_rules() + (web_ui_rules() if with_web_ui else []),
-            converters={'ext': PemExtensionConverter}
+            converters={'ext': PemExtensionConverter},
         )
 
         handlers: Dict[str, Callable] = {
@@ -242,17 +296,20 @@ class Animator:
             'certs': self.serve_cert,
             'attr-certs': self.serve_attr_cert,
             'attr-certs-by-holder': self.serve_attr_certs_of_holder,
-            'plugin': self.serve_plugin
+            'plugin': self.serve_plugin,
         }
 
         if with_web_ui:
             self.index_html = gen_index(iter(architectures))
-            handlers.update({
-                'any-cert': self.serve_any_cert,
-                'any-attr-cert': self.serve_any_attr_cert,
-                'attr-certs-of': self.serve_all_attr_certs_of_holder,
-                'cert-bundle': self.serve_zip, 'pfx-download': self.serve_pfx
-            })
+            handlers.update(
+                {
+                    'any-cert': self.serve_any_cert,
+                    'any-attr-cert': self.serve_any_attr_cert,
+                    'attr-certs-of': self.serve_all_attr_certs_of_holder,
+                    'cert-bundle': self.serve_zip,
+                    'pfx-download': self.serve_pfx,
+                }
+            )
 
         self._handlers = handlers
 
@@ -273,8 +330,9 @@ class Animator:
         response = ocsp_resp.build_ocsp_response(req)
         return Response(response.dump(), mimetype='application/ocsp-response')
 
-    def serve_timestamp_response(self, request: Request, *,
-                                 label: str, arch: str):
+    def serve_timestamp_response(
+        self, request: Request, *, label: str, arch: str
+    ):
         pki_arch = self.architectures[ArchLabel(arch)]
         tsa = pki_arch.service_registry.summon_timestamper(
             ServiceLabel(label), self.at_time(request)
@@ -284,8 +342,15 @@ class Animator:
         response = tsa.request_tsa_response(req)
         return Response(response.dump(), mimetype='application/timestamp-reply')
 
-    def serve_crl(self, request: Request, *,
-                  label: ServiceLabel, arch: str, crl_no, use_pem):
+    def serve_crl(
+        self,
+        request: Request,
+        *,
+        label: ServiceLabel,
+        arch: str,
+        crl_no,
+        use_pem,
+    ):
         pki_arch = self.architectures[ArchLabel(arch)]
         mime = 'application/x-pem-file' if use_pem else 'application/pkix-crl'
         if crl_no is not None:
@@ -300,8 +365,9 @@ class Animator:
             data = pem.armor('X509 CRL', data)
         return Response(data, mimetype=mime)
 
-    def serve_any_cert(self, _request: Request, *,
-                       arch: str, label: str, use_pem):
+    def serve_any_cert(
+        self, _request: Request, *, arch: str, label: str, use_pem
+    ):
         mime = 'application/x-pem-file' if use_pem else 'application/pkix-cert'
         pki_arch = self.architectures[ArchLabel(arch)]
         cert = pki_arch.get_cert(CertLabel(label))
@@ -311,11 +377,13 @@ class Animator:
             data = pem.armor('certificate', data)
         return Response(data, mimetype=mime)
 
-    def serve_any_attr_cert(self, _request: Request, *,
-                            arch: str, label: str, use_pem):
+    def serve_any_attr_cert(
+        self, _request: Request, *, arch: str, label: str, use_pem
+    ):
         mime = (
             'application/x-pem-file'
-            if use_pem else 'application/pkix-attr-cert'
+            if use_pem
+            else 'application/pkix-attr-cert'
         )
         pki_arch = self.architectures[ArchLabel(arch)]
         cert = pki_arch.get_attr_cert(CertLabel(label))
@@ -325,14 +393,21 @@ class Animator:
             data = pem.armor('attribute certificate', data)
         return Response(data, mimetype=mime)
 
-    def serve_cert(self, _request: Request, *, label: str, arch: str,
-                   cert_label: Optional[str], use_pem):
+    def serve_cert(
+        self,
+        _request: Request,
+        *,
+        label: str,
+        arch: str,
+        cert_label: Optional[str],
+        use_pem,
+    ):
         mime = 'application/x-pem-file' if use_pem else 'application/pkix-cert'
         pki_arch = self.architectures[ArchLabel(arch)]
-        cert_label = CertLabel(cert_label) if cert_label is not None else None
+        cert_lbl = CertLabel(cert_label) if cert_label is not None else None
 
         cert = pki_arch.service_registry.get_cert_from_repo(
-            ServiceLabel(label), cert_label
+            ServiceLabel(label), cert_lbl
         )
         if cert is None:
             raise NotFound()
@@ -342,8 +417,15 @@ class Animator:
             data = pem.armor('certificate', data)
         return Response(data, mimetype=mime)
 
-    def serve_attr_cert(self, _request: Request, *, label: str, arch: str,
-                        cert_label: Optional[str], use_pem):
+    def serve_attr_cert(
+        self,
+        _request: Request,
+        *,
+        label: str,
+        arch: str,
+        cert_label: Optional[str],
+        use_pem,
+    ):
         pki_arch = self.architectures[ArchLabel(arch)]
         svc_reg = pki_arch.service_registry
         svc_label = ServiceLabel(label)
@@ -360,7 +442,8 @@ class Animator:
         else:
             mime = (
                 'application/x-pem-file'
-                if use_pem else 'application/pkix-attr-cert'
+                if use_pem
+                else 'application/pkix-attr-cert'
             )
             cert = svc_reg.get_attr_cert_from_repo(
                 svc_label, CertLabel(cert_label)
@@ -372,8 +455,9 @@ class Animator:
         if use_pem:
             data = pem.armor(
                 'attribute certificate'
-                if cert_label is not None else 'certificate',
-                data
+                if cert_label is not None
+                else 'certificate',
+                data,
             )
         return Response(data, mimetype=mime)
 
@@ -382,19 +466,15 @@ class Animator:
         data_buf = BytesIO()
         for cert_spec in cert_specs:
             cert = pki_arch.get_attr_cert(cert_spec.label)
-            data_buf.write(
-                pem.armor(
-                    'attribute certificate',
-                    cert.dump()
-                )
-            )
+            data_buf.write(pem.armor('attribute certificate', cert.dump()))
         data = data_buf.getvalue()
         if not data:
             raise NotFound()
         return data
 
-    def serve_all_attr_certs_of_holder(self, _request: Request, *,
-                                       arch: str, entity_label: str):
+    def serve_all_attr_certs_of_holder(
+        self, _request: Request, *, arch: str, entity_label: str
+    ):
         pki_arch = self.architectures[ArchLabel(arch)]
         cert_specs = pki_arch.enumerate_attr_certs_of_holder(
             EntityLabel(entity_label),
@@ -402,27 +482,30 @@ class Animator:
         data = self._build_attr_cert_payload(pki_arch, cert_specs)
         return Response(data, mimetype='application/pkix-attr-cert')
 
-    def serve_attr_certs_of_holder(self, _request: Request, *,
-                                   label: str, arch: str, entity_label: str):
+    def serve_attr_certs_of_holder(
+        self, _request: Request, *, label: str, arch: str, entity_label: str
+    ):
         pki_arch = self.architectures[ArchLabel(arch)]
         svc_label = ServiceLabel(label)
         info = pki_arch.service_registry.get_attr_cert_repo_info(svc_label)
         if not info.publish_by_holder:
             raise NotFound()
         cert_specs = pki_arch.enumerate_attr_certs_of_holder(
-            EntityLabel(entity_label), info.for_issuer,
+            EntityLabel(entity_label),
+            info.for_issuer,
         )
         data = self._build_attr_cert_payload(pki_arch, cert_specs)
         return Response(data, mimetype='application/pkix-attr-cert')
 
-    def serve_plugin(self, request: Request, plugin_label: str, *, label: str,
-                     arch: str):
+    def serve_plugin(
+        self, request: Request, plugin_label: str, *, label: str, arch: str
+    ):
         pki_arch = self.architectures[ArchLabel(arch)]
         services = pki_arch.service_registry
-        plugin_label = PluginLabel(plugin_label)
-        label = ServiceLabel(label)
+        plugin_lbl = PluginLabel(plugin_label)
+        svc_lbl = ServiceLabel(label)
         try:
-            plugin_info = services.get_plugin_info(plugin_label, label)
+            plugin_info = services.get_plugin_info(plugin_lbl, svc_lbl)
         except ConfigurationError:
             raise NotFound()
 
@@ -430,7 +513,7 @@ class Animator:
         req_content = request.stream.read()
         try:
             response_bytes = services.invoke_plugin(
-                plugin_label, label, req_content, at_time=self.at_time(request)
+                plugin_lbl, svc_lbl, req_content, at_time=self.at_time(request)
             )
         except PluginServiceRequestError as e:
             raise BadRequest(e.user_msg)
@@ -446,8 +529,11 @@ class Animator:
         zip_buffer.seek(0)
         data = zip_buffer.read()
         cd_header = f'attachment; filename="{arch}-certificates.zip"'
-        return Response(data, mimetype='application/zip',
-                        headers={'Content-Disposition': cd_header})
+        return Response(
+            data,
+            mimetype='application/zip',
+            headers={'Content-Disposition': cd_header},
+        )
 
     def serve_pfx(self, request: Request, *, arch):
         pki_arch = self.architectures[ArchLabel(arch)]
@@ -456,16 +542,21 @@ class Animator:
         except KeyError:
             raise BadRequest()
 
-        cert = CertLabel(cert)
-        if not (pyca_cryptography_present() and
-                pki_arch.is_subject_key_available(cert)):
+        cert_label = CertLabel(cert)
+        if not (
+            pyca_cryptography_present()
+            and pki_arch.is_subject_key_available(cert_label)
+        ):
             raise NotFound()
 
         pass_bytes = request.form.get('passphrase', '').encode('utf8')
-        data = pki_arch.package_pkcs12(cert, password=pass_bytes or None)
-        cd_header = f'attachment; filename="{cert}.pfx"'
-        return Response(data, mimetype='application/x-pkcs12',
-                        headers={'Content-Disposition': cd_header})
+        data = pki_arch.package_pkcs12(cert_label, password=pass_bytes or None)
+        cd_header = f'attachment; filename="{cert_label}.pfx"'
+        return Response(
+            data,
+            mimetype='application/x-pkcs12',
+            headers={'Content-Disposition': cd_header},
+        )
 
     def dispatch(self, request: Request):
         adapter = self.url_map.bind_to_environ(request.environ)
@@ -519,12 +610,15 @@ class LazyAnimator:
         )
 
         cfg = CertomancerConfig.from_file(
-            cfg_file, key_search_dir=key_dir, config_search_dir=config_dir,
-            allow_external_config=extl_config
+            cfg_file,
+            key_search_dir=key_dir,
+            config_search_dir=config_dir,
+            allow_external_config=extl_config,
         )
         self.animator = Animator(
-            AnimatorArchStore(cfg.pki_archs), with_web_ui=with_web_ui,
-            allow_time_override=allow_time_override
+            AnimatorArchStore(cfg.pki_archs),
+            with_web_ui=with_web_ui,
+            allow_time_override=allow_time_override,
         )
 
     def __call__(self, environ, start_response):
